// 测试使用
// import { Device, Loading, Toast, locale } from 'seedsui-react'
// import Preview from 'seedsui-react/lib/Preview/instance.js'
// 内库使用
import Device from "./../Device";
import Loading from "./../Loading";
import Toast from "./../Toast";
import locale from "./../locale";
import Preview from "./../Preview/instance.js";
var Bridge = {
  /**
   * 定制功能
   */
  platform: 'browser',
  // 自定义操作
  invoke: function invoke() {
    Toast.show({
      content: locale('invoke仅可在微信或APP中使用', 'SeedsUI_only_app_wechat', ['invoke'])
    });
  },
  // 配置鉴权
  init: function init(cb) {
    if (typeof cb === 'function') cb({
      errMsg: 'config:ok'
    });
  },
  // 判断是否是主页
  isHomePage: function isHomePage(callback, rule) {
    if (rule && window.location.href.indexOf(rule) >= 0) {
      callback(true);
      return;
    }
    callback(false);
  },
  // 获得版本信息
  getAppVersion: function getAppVersion() {
    return '';
  },
  // 回到主页
  goHome: function goHome() {
    window.history.go(-1);
  },
  // 退出到登陆页面
  logOut: function logOut() {
    console.log('logOut方法仅在app上工作');
  },
  // 打开新的窗口
  openWindow: function openWindow() {
    var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var url = params.url;
    if (url.indexOf('h5:') === 0) url = url.replace(/^h5:/, '');else if (url.indexOf('webview:') === 0) url = url.replace(/^webview:/, '');
    if (params.target === '_self') {
      window.location.replace(url);
    }
    if (Device.device === 'pc') {
      window.open(url);
      return;
    }
    if (url) window.location.href = url;
  },
  // 关闭窗口
  closeWindow: function closeWindow() {
    window.history.go(-1);
  },
  // setTitle: '已在base中实现'
  // 返回监听
  onHistoryBack: function onHistoryBack() {
    Toast.show({
      content: locale('onHistoryBack仅可在企业微信或APP中使用', 'SeedsUI_only_app_wechat', ['onHistoryBack'])
    });
  },
  // 导航
  openLocation: function openLocation() {
    Toast.show({
      content: locale('openLocation仅可在企业微信或APP中使用', 'SeedsUI_only_app_wechat', ['openLocation'])
    });
  },
  /**
   * 扫描二维码并返回结果
   * @param {Object} params
   * @return {Object} {resultStr: ''}
   */
  scanQRCode: function scanQRCode() {
    var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var self = this;
    if (!self.debug) {
      Toast.show({
        content: locale('此功能仅可在微信或APP中使用', 'SeedsUI_only_app_wechat', ['scanQRCode'])
      });
      if (params.fail) params.fail({
        errMsg: "scanQRCode:".concat(locale('扫码失败', 'SeedsUI_scancode_failed'), ", ").concat(locale('请稍后重试', 'SeedsUI_try_again_later'))
      });
      return;
    }
    setTimeout(function () {
      if (params.success) params.success({
        resultStr: '504823170310092750280333'
      });
    }, 500);
  },
  // 拍照、本地选图
  chooseImage: function chooseImage() {
    var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var self = this;
    if (!self.debug) {
      Toast.show({
        content: locale('chooseImage仅可在微信或APP中使用', 'SeedsUI_only_app_wechat', ['chooseImage'])
      });
      return;
    }
    var res = {
      sourceType: 'camera',
      // 微信返回的两种来源: 'camera', 'album'
      errMsg: 'chooseImage:ok',
      localIds: ['https://static.zcool.cn/git_z/z/common/images/svg/logo.svg', 'https://static.zcool.cn/v3.5.180706.5/zcool/client/image/logo.png']
    };
    if (params.success) params.success(res);
  },
  // 上传图片
  uploadImage: function uploadImage() {
    var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var self = this;
    if (!self.debug) {
      Toast.show({
        content: locale('uploadImage仅可在微信或APP中使用', 'SeedsUI_only_app_wechat', ['uploadImage'])
      });
      return;
    }
    Loading.show();
    setTimeout(function () {
      Loading.hide();
      Toast.show({
        content: locale('上传完成', 'SeedsUI_upload_finished')
      });
      var res = {
        errMsg: 'uploadImage:ok',
        mediaUrl: '',
        serverId: new Date().getTime()
      };
      if (params.success) params.success(res);
    }, 1000);
  },
  // 图片预览
  // @params {urls:'需要预览的图片http链接列表',index:'图片索引',layerHTML:'图片上方的浮层'}
  preview: null,
  previewImage: function previewImage() {
    var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var self = this;
    if (!params.urls || !params.urls.length) {
      if (params.fail) params.fail({
        errMsg: 'previewImage:fail' + locale('没有预览图片地址', 'SeedsUI_previewimage_no_url')
      });
      return;
    }
    var src = params.urls[params.index || 0];
    if (!src) {
      if (params.fail) params.fail({
        errMsg: 'previewImage:fail' + locale('图片地址无效', 'SeedsUI_invalid_image_src')
      });
      return;
    }
    var layerHTML = params.layerHTML || '';
    if (!self.preview) {
      self.preview = new Preview({
        src: src,
        layerHTML: layerHTML,
        onSuccess: function onSuccess(s) {
          s.show();
          if (params.success) params.success(s);
        },
        onError: function onError() {
          if (params.fail) params.fail({
            errMsg: 'previewImage:fail' + locale('图片地址无效', 'SeedsUI_invalid_image_src')
          });
        }
      });
    } else {
      self.preview.updateParams({
        src: src,
        layerHTML: layerHTML
      });
    }
    return self.preview;
  },
  // 视频文件上传
  uploadFile: function uploadFile() {
    Toast.show({
      content: locale('uploadFile仅可在APP中使用', 'SeedsUI_only_app_wechat', ['uploadFile'])
    });
  },
  // debug:录像
  chooseVideo: function chooseVideo() {
    var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    console.log('chooseVideo方法在浏览器上无法运行');
    var res = {
      sourceType: 'camera',
      // 微信返回的两种来源: 'camera', 'album'
      errMsg: 'chooseVideo:ok',
      tempFilePath: 'http://res.waiqin365.com/video/v2001.MP4',
      duration: '',
      size: '',
      height: '',
      width: ''
    };
    if (params.success) params.success(res);
  },
  /**
   * 文件操作: 预览文件
   * @param {Object} params
   * params: {
   *  url: '', // 需要预览文件的地址(必填，可以使用相对路径)
   *  name: '', // 需要预览文件的文件名(不填的话取url的最后部分)
   *  size: 1048576 // 需要预览文件的字节大小(必填)
   * }
   */
  previewFile: function previewFile(params) {
    var self = this;
    if (!self.debug) {
      Toast.show({
        content: locale('previewFile仅可在微信或APP中使用', 'SeedsUI_only_app_wechat', ['previewFile'])
      });
      if (params.fail) params.fail({
        errMsg: "previewFile:fail".concat(locale('预览文件失败', 'SeedsUI_previewfile_failed'), ", ").concat(locale('请稍后重试', 'SeedsUI_try_again_later'))
      });
      return;
    }
    if (params.success) params.success({
      errMsg: "previewFile:ok".concat(locale('预览文件成功', 'SeedsUI_previewfile_success'))
    });
    if (params.url) window.location.href = params.url;
  }
};
export default Bridge;