var _excluded = ["visible", "value", "list", "multiple", "allowClear", "disabled", "onSelect", "onBeforeChange", "onChange", "defaultPickerValue", "titleFormatter", "type", "min", "max", "onError", "ranges", "modal", "separator"];
function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }
// require PrototypeDate.js和PrototypeString.js
import React, { forwardRef, useEffect, useRef, useImperativeHandle } from 'react';
import locale from "./../../locale";
// 测试使用
// import locale from 'seedsui-react/lib/locale'
import { getFormat } from "./../utils";
import Instance from "./instance.js";
import Utils from "./Utils";
var Main = /*#__PURE__*/forwardRef(function (_ref, ref) {
  var visible = _ref.visible,
    value = _ref.value,
    list = _ref.list,
    multiple = _ref.multiple,
    allowClear = _ref.allowClear,
    disabled = _ref.disabled,
    onSelect = _ref.onSelect,
    onBeforeChange = _ref.onBeforeChange,
    onChange = _ref.onChange,
    defaultPickerValue = _ref.defaultPickerValue,
    titleFormatter = _ref.titleFormatter,
    _ref$type = _ref.type,
    type = _ref$type === void 0 ? 'date' : _ref$type,
    min = _ref.min,
    max = _ref.max,
    onError = _ref.onError,
    ranges = _ref.ranges,
    modal = _ref.modal,
    separator = _ref.separator,
    props = _objectWithoutProperties(_ref, _excluded);
  // 过滤非法的类型
  if (!['year', 'quarter', 'month', 'date', 'time', 'datetime'].includes(type)) {
    console.error("DatePicker.Modal: Wrong parameter with \"type\"! You need correct to ['year', 'quarter', 'month', 'date', 'time', 'datetime'] any one");
    // eslint-disable-next-line
    type = 'date';
  }

  // 节点
  var mainRef = useRef(null);
  var instance = useRef(null);
  useImperativeHandle(ref, function () {
    return {
      rootDOM: mainRef.current,
      getRootDOM: function getRootDOM() {
        return mainRef.current;
      },
      instance: instance.current,
      getInstance: function getInstance() {
        return instance.current;
      },
      getValue: getValue,
      update: update,
      // 获取标题
      getTitle: function getTitle() {
        var _instance$current;
        if (instance !== null && instance !== void 0 && (_instance$current = instance.current) !== null && _instance$current !== void 0 && _instance$current.formatTitle) {
          var format = '';
          // 如果用户没有自定义标题格式, 则使用动态格式
          if (typeof titleFormatter === 'function') {
            format = titleFormatter({
              type: type,
              value: getValue(),
              ranges: ranges,
              separator: separator
            });
          }
          if (!format || typeof format !== 'string') {
            // 只有年月日、年月日时分才显示周几
            format = "".concat(getFormat(type)).concat(['date', 'datetime'].includes(type) ? ' d' : '');
          }
          return instance.current.formatTitle(format);
        }
        return '';
      }
    };
  });

  // useEffect(() => {
  //   initInstance()
  //   // eslint-disable-next-line
  // }, [])

  useEffect(function () {
    update();
    // eslint-disable-next-line
  }, [value, type]);

  // 更新句柄, 防止synchronization模式, 每次组件在render的时候都生成上次render的state、function、effects
  if (instance.current) {
    instance.current.params.onScrollTransitionEnd = handleChange;
  }

  // 更新视图
  function update() {
    if (instance.current) {
      instance.current.updateParams({
        viewType: type,
        yyUnit: locale('', 'SeedsUI_picker_unit_year'),
        QQUnit: locale('', 'SeedsUI_picker_unit_quarter'),
        MMUnit: locale('', 'SeedsUI_picker_unit_month'),
        ddUnit: locale('', 'SeedsUI_picker_unit_day'),
        HHUnit: locale('', 'SeedsUI_picker_unit_hour'),
        mmUnit: locale('', 'SeedsUI_picker_unit_minute')
      });
      var def = Utils.getDefaults(value, defaultPickerValue);
      instance.current.setDefaults(def);
      instance.current.update();
    } else {
      initInstance();
    }
  }

  // 滚动结束
  function handleChange(s) {
    setTimeout(function () {
      // 根据月份算日
      if ((s.params.viewType === 'date' || s.params.viewType === 'datetime') && (s.activeSlot.index === 0 || s.activeSlot.index === 1)) {
        var year = s.activeOptions[0]['id'];
        var month = s.activeOptions[1]['id'];
        var defaultDay = s.activeOptions[2]['id'];
        s.updateDays(year, month, defaultDay); // 更新总天数
      }

      // 触发onChange事件
      if (onChange) onChange(getValue());
    }, 300);
  }

  // 获取选中值
  function getValue() {
    var s = instance.current;
    // 获取选中项
    var newValue = s.getActiveDate(s.activeOptions);
    var activeKeys = s.activeOptions.map(function (n, i, a) {
      return n['id'];
    });
    s.setDefaultsByKeys(activeKeys);
    return newValue;
  }

  // 实例化
  function initInstance() {
    if (!mainRef || !mainRef.current) return;
    var data = Utils.getData(list);
    var def = Utils.getDefaults(value, defaultPickerValue);
    // render数据
    instance.current = new Instance({
      wrapper: mainRef.current,
      viewType: type,
      yearsData: data.yearsData,
      quartersData: data.quartersData,
      monthsData: data.monthsData,
      daysData: data.daysData,
      hoursData: data.hoursData,
      minutesData: data.minutesData,
      defaultYear: def.year,
      defaultQuarter: def.quarter,
      defaultMonth: def.month,
      defaultDay: def.day,
      defaultHour: def.hour,
      defaultMinute: def.minute,
      onScrollTransitionEnd: handleChange,
      yyUnit: locale('', 'SeedsUI_picker_unit_year'),
      // 年
      QQUnit: locale('', 'SeedsUI_picker_unit_quarter'),
      // 季
      MMUnit: locale('', 'SeedsUI_picker_unit_month'),
      // 月
      ddUnit: locale('', 'SeedsUI_picker_unit_day'),
      // 日
      HHUnit: locale('', 'SeedsUI_picker_unit_hour'),
      // 时
      mmUnit: locale('', 'SeedsUI_picker_unit_minute') // 分
    });
  }
  return /*#__PURE__*/React.createElement("div", _extends({}, props, {
    className: "picker-main".concat(disabled ? ' disabled' : '').concat(props.className ? ' ' + props.className : ''),
    ref: mainRef
  }), /*#__PURE__*/React.createElement("div", {
    className: "picker-layer"
  }, /*#__PURE__*/React.createElement("div", {
    className: "picker-layer-frame"
  })), /*#__PURE__*/React.createElement("div", {
    className: "picker-slotbox"
  }));
});
export default Main;