import locale from "./../locale";
import destroy from "./destroy";

// 弹出对话框
export default function confirm(_ref) {
  var portal = _ref.portal,
    maskClosable = _ref.maskClosable,
    onVisibleChange = _ref.onVisibleChange,
    maskProps = _ref.maskProps,
    captionProps = _ref.captionProps,
    content = _ref.content,
    contentProps = _ref.contentProps,
    footerProps = _ref.footerProps,
    submitProps = _ref.submitProps,
    _ref$cancelProps = _ref.cancelProps,
    cancelProps = _ref$cancelProps === void 0 ? {} : _ref$cancelProps;
  var mask = null;

  // 确定按钮
  if (submitProps === undefined) {
    // eslint-disable-next-line
    submitProps = {
      onClick: function onClick() {}
    };
  }

  // 更新class和style
  function updateStyle(_ref2) {
    var target = _ref2.target,
      props = _ref2.props,
      baseClassName = _ref2.baseClassName;
    if (!target) return;
    var _ref3 = props || {},
      style = _ref3.style,
      className = _ref3.className;
    // 还原样式
    target.setAttribute('style', '');
    target.setAttribute('class', baseClassName);

    // 增加样式
    if (style) {
      for (var stylePropName in style) {
        target.style[stylePropName] = style[stylePropName];
      }
    }
    if (className) {
      target.className = "".concat(baseClassName).concat(className ? ' ' + className : '');
    }
  }

  // 更新属性
  function updateAttribute(mask) {
    var _submitProps;
    // 更新遮罩
    updateStyle({
      target: mask,
      props: maskProps,
      baseClassName: 'mask modal-mask'
    });

    // 更新标题
    var captionDOM = mask.querySelector('.modal-caption');
    updateStyle({
      target: captionDOM,
      props: captionProps,
      baseClassName: 'modal-caption'
    });
    if (captionProps !== null && captionProps !== void 0 && captionProps.caption) {
      captionDOM.classList.remove('hide');
      captionDOM.innerHTML = captionProps.caption;
    } else {
      captionDOM.classList.add('hide');
    }

    // 更新内容
    var contentDOM = mask.querySelector('.modal-content');
    updateStyle({
      target: contentDOM,
      props: contentProps,
      baseClassName: 'modal-content'
    });
    if (content) {
      contentDOM.innerHTML = content;
    } else {
      contentDOM.innerHTML = '';
    }

    // 更新底部
    var footerDOM = mask.querySelector('.modal-footer');
    updateStyle({
      target: footerDOM,
      props: footerProps,
      baseClassName: 'modal-footer'
    });

    // 提交按钮
    var submitDOM = mask.querySelector('.modal-ok');
    updateStyle({
      target: submitDOM,
      props: submitProps,
      baseClassName: 'modal-ok'
    });
    if (submitProps || (_submitProps = submitProps) !== null && _submitProps !== void 0 && _submitProps.caption) {
      var _submitProps2;
      submitDOM.classList.remove('hide');
      submitDOM.innerHTML = ((_submitProps2 = submitProps) === null || _submitProps2 === void 0 ? void 0 : _submitProps2.caption) || locale('确定', 'SeedsUI_ok');
    } else {
      submitDOM.classList.add('hide');
    }

    // 取消按钮
    var cancelDOM = mask.querySelector('.modal-cancel');
    updateStyle({
      target: cancelDOM,
      props: cancelProps,
      baseClassName: 'modal-cancel'
    });
    if (cancelProps || cancelProps !== null && cancelProps !== void 0 && cancelProps.caption) {
      cancelDOM.classList.remove('hide');
      cancelDOM.innerHTML = (cancelProps === null || cancelProps === void 0 ? void 0 : cancelProps.caption) || locale('取消', 'SeedsUI_cancel');
    } else {
      cancelDOM.classList.add('hide');
    }

    // 更新事件中用到的属性（否则事件中将永远读取到的是闭包中的属性，即上次的属性）
    mask.maskClosable = maskClosable;
    mask.submitProps = submitProps;
    mask.cancelProps = cancelProps;
    mask.onVisibleChange = onVisibleChange;

    // dom透传
    if (toString.call(portal).indexOf('HTML') !== -1) {
      portal.appendChild(mask);
    } else {
      ;
      (document.getElementById('root') || document.body).appendChild(mask);
    }
  }

  // 点击遮罩
  function handleMaskClick(e) {
    e.stopPropagation();

    // 点击确定或取消按钮
    if (e.target.classList.contains('modal-cancel') || e.target.classList.contains('modal-ok')) {
      handleButtonClick(e);
      return;
    }

    // 点击遮罩
    if (e.target.classList.contains('mask')) {
      var _mask, _mask2;
      // 读取更新后的属性
      var _maskClosable = (_mask = mask) === null || _mask === void 0 ? void 0 : _mask.maskClosable;
      var _onVisibleChange = (_mask2 = mask) === null || _mask2 === void 0 ? void 0 : _mask2.onVisibleChange;
      if (_maskClosable) {
        if (_onVisibleChange) _onVisibleChange(false);
        destroy(e.currentTarget);
      }
    }
  }

  // 点击确定或者取消按钮
  function handleButtonClick(e) {
    var _mask3, _mask4, _mask5;
    var isOk = e.target.classList.contains('modal-ok');

    // 读取更新后的属性
    var buttonProps = isOk ? (_mask3 = mask) === null || _mask3 === void 0 ? void 0 : _mask3.submitProps : (_mask4 = mask) === null || _mask4 === void 0 ? void 0 : _mask4.cancelProps;
    var onVisibleChange = (_mask5 = mask) === null || _mask5 === void 0 ? void 0 : _mask5.onVisibleChange;
    if (typeof (buttonProps === null || buttonProps === void 0 ? void 0 : buttonProps.onClick) === 'function') {
      var _buttonProps$onClick;
      var close = (_buttonProps$onClick = buttonProps.onClick()) !== null && _buttonProps$onClick !== void 0 ? _buttonProps$onClick : true;
      if (close) {
        if (onVisibleChange) onVisibleChange(false);
        destroy(e.currentTarget.closest('.modal-mask'));
      }
    } else {
      if (onVisibleChange) onVisibleChange(false);
      destroy(e.currentTarget.closest('.modal-mask'));
    }
    e.stopPropagation();
  }

  // 渲染
  function render() {
    var modalId = '__SeedsUI_modal_el__';
    // 如果没生成成功, 则强制生成
    var container = document.body;
    if (toString.call(portal).indexOf('HTML') !== -1) {
      container = portal;
    }
    mask = container.querySelector('#' + modalId);
    if (!mask) {
      // 创建dom
      mask = document.createElement('div');
      mask.setAttribute('class', "mask modal-mask");
      mask.setAttribute('id', modalId);
      mask.innerHTML = "\n        <div class=\"popup-animation modal modal-alert middle\" data-animation=\"zoom\">\n          <div class=\"modal-body\">\n            <div class=\"modal-caption hide\"></div>\n            <div class=\"modal-content\"></div>\n          </div>\n          <div class=\"modal-footer\">\n            <div class=\"modal-cancel\"></div>\n            <div class=\"modal-ok\"></div>\n          </div>\n        </div>\n      "

      // 添加到dom上
      ;
      (container || document.getElementById('root') || document.body).appendChild(mask);

      // 绑定事件
      attach(mask);
    }

    // 更新属性
    updateAttribute(mask);

    // 渲染完成后补充active, 解决渲染后动画不生效的问题
    setTimeout(function () {
      mask = container.querySelector('#' + modalId);
      if (!mask) return;
      // 如果正在移除，则停止移除
      if (mask.timeout) {
        window.clearTimeout(mask.timeout);
      }
      // 动画显示
      mask.classList.add('active');
      mask.querySelector('.modal-alert').classList.add('active');
    }, 10);
    return mask;
  }

  // 绑定事件
  function attach(mask) {
    // 绑定事件
    mask.removeEventListener('click', handleMaskClick, false);
    mask.addEventListener('click', handleMaskClick, false);
  }

  // 渲染与绑定事件
  render();
  return mask;
}